package tools.descartes.prisma.agent.wildfly.callstack.impl;

import tools.descartes.prisma.agent.wildfly.callstack.InterfaceRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationGroupRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.TraceVisitor;
import tools.descartes.prisma.agent.wildfly.parameters.ValueCharacterization;
import tools.descartes.prisma.agent.wildfly.service.ModelExtractionService;

public class DefaultOperationRecord extends OperationRecord {
	
	private ValueCharacterization value = null;
	private String[] observedParameters = new String[0];
	private int[] observedParametersIndeces = new int[0];
	private InvocationGroupRecord[] groups = new InvocationGroupRecord[1];
	
	public DefaultOperationRecord(InterfaceRecord interf, String operation, ValueCharacterization value) {
		super(interf, operation);
		this.value = value;
		if (value != null) {
			observedParameters = new String[1];
			observedParametersIndeces = new int[1]; 
			observedParameters[0] = value.getParameterName();
			observedParametersIndeces[0] = value.getParameterIndex();
			groups = new InvocationGroupRecord[value.getNumberOfValues()];
		} else {
			groups[0] = new DefaultInvocationGroupRecord(this, "");
			ModelExtractionService.INSTANCE.newInvocationGroupRecord(groups[0]);
		}
	}
	
	@Override
	public String[] getObservedParameterNames() {
		return observedParameters;
	}
	
	@Override
	public int[] getObservedParameterIndeces() {
		return observedParametersIndeces;
	}
	
	@Override
	public int characterizeValue(int param, Object value) {
		if (param == 0 && value != null) {
//			System.out.println("Characterize value: " + param + " " + value);
			return this.value.getValueIndex(value);
		}
		return 0;
	}
	
	
	@Override
	public boolean isFiltered() {
		return false;
	}
	
	@Override
	public InvocationGroupRecord[] getInvocationGroups() {
		return groups;
	}
	
	@Override
	public void accept(TraceVisitor visitor, InvocationGroupRecord source) {
//		System.out.println("Visit operation " + getOperationName());
		if (source != null) {
			visitor.visitRequiredOperation(source, this);
		} else {
			visitor.visitOperation(this);
		}
		for (InvocationGroupRecord rec : groups) {
			if (rec != null) {
				rec.accept(visitor, source);
			}
		}
	}
	
	@Override
	public InvocationGroupRecord requireInvocationGroup(int idx) {
		if (groups[idx] == null && value != null) {
			groups[idx] = new DefaultInvocationGroupRecord(this, value.getValueIdentifier(idx));
			ModelExtractionService.INSTANCE.newInvocationGroupRecord(groups[idx]);
		}
		return groups[idx];
	}
}
