package tools.descartes.prisma.agent.wildfly.service;

import org.jboss.dmr.ModelNode;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;

import tools.descartes.prisma.agent.wildfly.extension.SubsystemExtension;
import tools.descartes.prisma.agent.wildfly.filters.ComponentPredicate;
import tools.descartes.prisma.agent.wildfly.filters.FilteredTraceFactory;
import tools.descartes.prisma.agent.wildfly.filters.InterfacePredicate;
import tools.descartes.prisma.agent.wildfly.filters.OperationPredicate;

public class FilterService implements Service<FilterService>  {
	
	private ModelNode definition;
	
	public FilterService(ModelNode definition) {
		this.definition = definition;
	}
	
	@Override
	public FilterService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}
	
	public void addPredicate(FilteredTraceFactory factory) {
		ModelNode compPattern = definition.get(SubsystemExtension.COMPONENT_PATTERN);
		ModelNode intPattern = definition.get(SubsystemExtension.INTERFACE_PATTERN);
		ModelNode opPattern = definition.get(SubsystemExtension.OPERATION_PATTERN);
		System.err.println("FilterService: " + compPattern.asString() + " " + intPattern.asString() + " " + opPattern.asString());
		
		if (!opPattern.asString().equals(".*")) {
			factory.addPredicate(new OperationPredicate(compPattern.asString(), intPattern.asString(), opPattern.asString()));
		} else if (!intPattern.asString().equals(".*")) {
			factory.addPredicate(new InterfacePredicate(compPattern.asString(), intPattern.asString()));
		} else if (!compPattern.asString().equals(".*")) {
			factory.addPredicate(new ComponentPredicate(compPattern.asString()));
		}		
	}

	@Override
	public void start(StartContext arg0) throws StartException {
		System.err.println("FitlerService started: " + definition.toJSONString(true));
	}

	@Override
	public void stop(StopContext arg0) {
		System.err.println("FitlerService stopped");
	}
}
