package tools.descartes.prisma.core.scopes;

import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.core.Transaction;
import tools.descartes.prisma.core.exceptions.CommitException;
import tools.descartes.prisma.model.skeleton.ModelSkeleton;

public abstract class Scope {
	
	private final String name;
	private final ModelRepository repository;
	
	public Scope(String name, ModelRepository repository) {
		this.name = name;
		this.repository = repository;
	}
	
	public String getName() {
		return name;
	}
	
	public ModelRepository getModelRepository() {
		return repository;
	}
	
	public String getFolder() {
		if (getBasePath().isEmpty()) {
			return getName();
		} else {
			return getBasePath() + "/" + getName();
		}
	}

	public boolean create() {
		try (Transaction transaction = getModelRepository().createTransaction()) {
			/*
			 * IMPORTANT: Lock the root object to avoid conflicts by agents who
			 * concurrently create scopes
			 */
			if (!getModelRepository().exists(getFolder())) {
				getModelRepository().createFolders(transaction, getFolder());
			}

			createResources(transaction);
			
			try {
				transaction.commit();
				return true;
			} catch (CommitException e) {
				e.printStackTrace();
			}
			return false;
		}
	}

	public void waitForCreation() {
		ModelRepository repository = getModelRepository();
		while (!repository.exists(getFolder())) {
			repository.waitForUpdate();
		}
	}

	public abstract void synchronizeModelSkeleton(Transaction transaction, ModelSkeleton skeleton);

	protected abstract void createResources(Transaction transaction);

	protected abstract String getBasePath();
}
